// Skladiste.java - Rukovanje skladistem.

abstract class Predmet {
  private static int ukBroj;
  private int idBroj = ++ ukBroj;

  public int broj () { return idBroj; }

  public abstract double V ();

  public abstract double Q ();

  public abstract Predmet kopija ();

  final public String vrsta () { return getClass ().getName (); }

  public String toString () { return idBroj + " " + vrsta (); }
}

abstract class Telo extends Predmet {
  protected double ro;
  
  public Telo (double roo) { ro = roo; }

  public final double Q () { return ro * V (); }

  public String toString () { return super.toString () + " " + ro; }
}

class Sfera extends Telo {
  private double r;

  public Sfera (double roo, double rr) { super (roo); r = rr; }

  public final double V () { return 4./3 * r*r*r * Math.PI; }

  public String toString () { return super.toString () + " " + r; }

  public Predmet kopija () { return new Sfera (ro, r); }
}

class Kvadar extends Telo {
  private double a, b, c;

  public Kvadar (double roo, double aa, double bb, double cc)
    { super (roo); a = aa; b = bb; c = cc; }

  public final double V () { return a * b * c; }

  public String toString ()
    { return super.toString () + " " + a + " " + b + " " + c; }

  public Predmet kopija () { return new Kvadar (ro, a, b, c); }
}

class Lista {

  private static class Elem {
    Predmet p; Elem sled;
    Elem (Predmet pp, Elem ss) { p = pp; sled = ss; }
    Elem (Predmet pp) { this (pp, null); }
  }

  private Elem prvi, posl, tek;
  private double zapr, tez;

  public void stavi (Predmet p) {
    Elem novi = new Elem (p);
    if (prvi == null) prvi = novi; else posl.sled = novi;
    posl = novi;
    zapr += p.V ();
    tez  += p.Q ();
  }

  public Predmet uzmi (int broj) {
    Elem tek = prvi, preth = null;
    while (tek!=null && tek.p.broj()!=broj) { preth=tek; tek=tek.sled; }
    if (tek != null) {
      if (preth == null) prvi = tek.sled; else preth.sled = tek.sled;
      if (tek.sled  == null) posl = preth;
      zapr -= tek.p.V ();
      tez  -= tek.p.Q ();
      return tek.p;
    } else return null;
  }

  public double V () { return zapr; }

  public double Q () { return tez;  }

  public Lista kopija () {
    Lista lst = new Lista ();
    for (Elem tek=prvi; tek!=null; tek=tek.sled)
      lst.stavi (tek.p.kopija());
    return lst;
  }

  public void naPrvi () { tek = prvi; }

  public Predmet dohvatiSledeci () {
    if (tek != null) { Predmet p = tek.p; tek = tek.sled; return p; }
      else return null;
  }
}

class Sklop extends Predmet {
  private Lista lst = new Lista ();

  public void stavi (Predmet p) { lst.stavi (p); }

  public Predmet uzmi (int broj) { return lst.uzmi (broj); }

  public final double V () { return lst.V (); }

  public final double Q () { return lst.Q (); }

  public Predmet kopija ()
    { Sklop s = new Sklop (); s.lst = lst.kopija (); return s; }

  private static int nivo = 0;

  private static String uvuci () { 
    StringBuffer s = new StringBuffer (); 
    for (int i=0; i<nivo; i++) s.append ("  "); 
    return s.toString (); 
  }

  public String toString () {
    StringBuffer s = new StringBuffer (uvuci ());
    s.append (broj ()).append (' ').append (vrsta ()).append (" {\n");
    nivo++; Predmet p;
    for (lst.naPrvi(); (p=lst.dohvatiSledeci())!=null;) {
      if (!(p instanceof Sklop)) s.append (uvuci ());
      s.append (p);
      if (!(p instanceof Sklop)) s.append ('\n');
    }
    nivo--;
    return s.append (uvuci ()).append ("}\n").toString ();
  }
}

class Skladiste {
  private static int ukBroj;
  private int idBroj = ++ ukBroj;
  private Lista lst = new Lista ();
  private double maxV, maxQ;

  Skladiste (double Vmax, double Qmax) { maxV = Vmax; maxQ = Qmax; }

  public int broj () { return idBroj; }

  public void stavi (Predmet p) throws ESkladZapremina, ESkladTezina {
    if (lst.V() + p.V() > maxV) throw new ESkladZapremina (p, this);
    if (lst.Q() + p.Q() > maxQ) throw new ESkladTezina    (p, this);
    lst.stavi (p);
  }

  public Predmet uzmi (int broj) throws ESkladNema { 
    Predmet p = lst.uzmi (broj);
    if (p == null) throw new ESkladNema (broj, this);
    return p;
  }

  public double V () { return lst.V (); }

  public double Q () { return lst.Q (); }

  public double Vmax () { return maxV; }

  public double Qmax () { return maxQ; }

  public String toString () {
    StringBuffer s = new StringBuffer (); Predmet p;
    for (lst.naPrvi(); (p=lst.dohvatiSledeci())!=null;)
      s.append (p.broj()).append (' ').append (p.vrsta()).append ('\n');
    return s.toString ();
  }
}

class ESkladNema extends Exception {
  public ESkladNema (int broj, Skladiste s) {
    super ("Ne postoji: predmet #" + broj + "; skladiste #" +
           s.broj ());
  }
}

class ESkladZapremina extends Exception {
  public ESkladZapremina (Predmet p, Skladiste s) {
    super ("Prevelika zapremina: predmet #" + p.broj() +
           " (V=" + p.V() + "); skladiste #" + s.broj() +
           " (Vmax=" + s.Vmax() + " Vprazno=" + (s.Vmax()-s.V()) + ")");
  }
}

class ESkladTezina extends Exception {
  public ESkladTezina (Predmet p, Skladiste s) {
    super ("Prevelika tezina: predmet #" + p.broj() +
           " (Q=" + p.Q() + "); skladiste #" + s.broj() +
           " (Qmax=" + s.Qmax() + " Qprazno=" + (s.Qmax()-s.Q()) + ")");
  }
}

class Skladista {

  private static void uvuci (int nivo) {
    for (int i=0; i<nivo; i++) System.out.print ("  ");
  }

  private static Predmet citaj (int nivo) {
    uvuci (nivo);
    System.out.print ("Vrsta (s, k, *)? ");
    switch (Citaj.Char ()) {
      case 's': case 'S': {
        Skladista.uvuci (nivo);
        System.out.print ("Gustina?     "); double ro = Citaj.Double ();
        Skladista.uvuci (nivo);
        System.out.print ("Poluprecnik? "); double r  = Citaj.Double ();
        return new Sfera (ro, r);
     }case 'k': case 'K': {
        Skladista.uvuci (nivo);
        System.out.print ("Gustina?     "); double ro = Citaj.Double ();
        Skladista.uvuci (nivo);
        System.out.print ("Ivice?       ");
        double a = Citaj.Double(), b = Citaj.Double(), c = Citaj.Double();
        return new Kvadar (ro, a, b, c);
     }case '*': {
        Sklop s = new Sklop (); Predmet p;
        do {
          p = citaj (nivo+1); if (p != null) s.stavi (p);
        } while (p != null);
        return s;
     }default: return null;
    }
  }

  public static void main (String[] vpar) {
    Skladiste[] skladista = new Skladiste [Integer.parseInt (vpar[0])];
    int s = 0; Predmet p = null;
    for (boolean kraj=false; !kraj; ) {
      System.out.print (
        "\nMoguce operacije su:\n\n"                    +
        "1 - stvaranje novog skladista\n"               +
        "2 - izbor skladista za rad\n"                  +
        "3 - citanje predmeta preko tastature\n"        +
        "4 - stavljanje predmeta u skladiste\n"         +
        "5 - uzimanje predmeta iz skladista\n"          +
        "6 - prebacivanje predmeta u drugo skladiste\n" +
        "7 - prikazivanje sadrzaja skladista\n"         +
        "8 - prikazivanje sadrzaja predmeta\n"          +
        "0 - zavrsetak rada\n\n"                        +
        "Unesite svoj izbor: "
      );
      char izb = Citaj.Char ();
      try {
        switch (izb) {
          case '1': {
            System.out.print ("Broj?      ");        s = Citaj.Int ();
            System.out.print ("Zapremina? "); double V = Citaj.Double ();
            System.out.print ("Nosivost?  "); double Q = Citaj.Double ();
            skladista[s-1] = new Skladiste (V, Q);
            break;
         }case '2': {
            System.out.print ("Broj?      "); s = Citaj.Int ();
            break;
         }case '3': {
            p = citaj (0);
            break;
         }case '4': {
            if (p == null)
              throw new Exception ("Ne postoji tekuci predmet");
            skladista[s-1].stavi (p.kopija ());
            break;
         }case '5': {
            System.out.print ("Broj predmeta? "); int b  = Citaj.Int ();
            p = skladista[s-1].uzmi (b);
            break;
         }case '6': {
            System.out.print ("Broj predmeta? "); int b  = Citaj.Int ();
            System.out.print ("U skladiste?   "); int s2 = Citaj.Int ();
            skladista[s2-1].stavi (skladista[s-1].uzmi (b));
            break;
         }case '7': {
            System.out.print ("\n" + skladista[s-1] + '\n');
            break;
         }case '8': {
            System.out.print ("\n" + p + '\n');
            break;
         }case '0': {
            kraj = true;
            break;
         }default: throw new Exception ("Nedozvoljeni izbor");
        }
      } catch (Exception g) {
        System.out.println ("\n*** " + g + " ***");
      }
    }
  }
}
