// Vektor.java - Klasa vektora sa paralelnim izracunavanjima.

class Sabirac extends Thread {
  private double a, b, c;
  public Sabirac (double aa, double bb) { a = aa; b = bb; }
  public void run () { c = a + b; }
  public double rezultat () { return c; }
}

class Mnozac extends Thread {
  private double a, b, c;
  public Mnozac (double aa, double bb) { a = aa; b = bb; }
  public void run () { c = a * b; }
  public double rezultat () { return c; }
}

class EIndeks extends Exception {
  private int ind;
  public EIndeks (int i) { ind = i; }
  public String toString ()
    { return "\n*** Nedozvoljeni indeks " + ind + " ***\n"; }
}

class EDuzina extends Exception {
  private int duz1, duz2;
  public EDuzina (int d1, int d2) { duz1 = d1; duz2 = d2; }
  public String toString ()
    { return "\n*** Neusaglasene duzine " + duz1 + " i " + duz2 + " ***\n"; }
}

class Vektor {
  private double[] niz;
  public Vektor (int duz) { niz = new double [duz]; }
  public void stavi (int ind, double broj) throws EIndeks {
    if (ind < 0 || ind >= niz.length) throw new EIndeks (ind);
    niz[ind] = broj;
  }

  public double uzmi (int ind) throws EIndeks {
    if (ind < 0 || ind >= niz.length) throw new EIndeks (ind);
    return niz[ind];
  }

  public String toString () {
    String s = "(";
    for (int i=0; i<niz.length; i++)
      { s += niz[i]; if (i < niz.length - 1) s += ","; }
    return s + ")";
  }

  public static Vektor zbir (Vektor v1, Vektor v2) throws EDuzina {
    int n = v1.niz.length, n2 = v2.niz.length;
    if (n != n2) throw new EDuzina (n, n2);
    Sabirac[] sab = new Sabirac [n];
    for (int i=0; i<n; i++)
      (sab[i] = new Sabirac (v1.niz[i], v2.niz[i])).start ();
    Vektor v = new Vektor (n);
    for (int i=0; i<n; i++) 
      try { sab[i].join(); v.niz[i] = sab[i].rezultat (); }
        catch (InterruptedException e) {}
    return v;
  }

  public static double skalpro (Vektor v1, Vektor v2) throws EDuzina {
    int n = v1.niz.length, n2 = v2.niz.length;
    if (n != n2) throw new EDuzina (n, n2);
    Mnozac[] mno = new Mnozac [n];
    for (int i=0; i<n; i++)
      (mno[i] = new Mnozac (v1.niz[i], v2.niz[i])).start ();
    double s = 0;
    for (int i=0; i<n; i++) 
      try { mno[i].join(); s += mno[i].rezultat (); }
        catch (InterruptedException e) {}
    return s;
  }

  public static void main (String[] vpar) {
    while (true) {
      try {
        System.out.print ("n1? "); int n1 = Citaj.Int ();
    if (n1 < 0) break;
        System.out.print ("v1? "); Vektor v1 = new Vektor (n1);
        for (int i=0; i<n1; v1.stavi(i++,Citaj.Double()));
        System.out.print ("n2? "); int n2 = Citaj.Int ();
    if (n2 < 0) break;
        System.out.print ("v2? "); Vektor v2 = new Vektor (n2);
        for (int i=0; i<n2; v2.stavi(i++,Citaj.Double()));
        System.out.println ("Zbir: " + Vektor.zbir(v1,v2));
        System.out.println ("Skalpro: " + Vektor.skalpro(v1,v2) + "\n");
      } catch (EIndeks g) {
        System.out.println (g);
      } catch (EDuzina g) {
        System.out.println (g);
      }
    }
  }

}  